/**
 * Codice ottenuto e revisionato dal sito:
 * http://www.developer.nokia.com/Community/Wiki/Controlling_the_camera_light_using_CCamera
 * 
 * (sono state apportate alcune modifiche per migliorare la compatibilità con S60 5th Edition 
 */

#include "flashlight.h"

CCameraFlashLight* CCameraFlashLight::NewL()
	{
	CCameraFlashLight* self = new (ELeave) CCameraFlashLight;
	CleanupStack::PushL(self);
	self->ConstructL();
	CleanupStack::Pop(); //self
	return self;
	}

void CCameraFlashLight::ConstructL()
	{
	iCamera = CCamera::New2L(*this, 0, 0);
	}

CCameraFlashLight::~CCameraFlashLight()
	{
	StopCapture();
	delete iCamera;
	}

void CCameraFlashLight::SetVideoLightL(TBool aOn)
	{
	iCamera->CameraInfo(iCameraInfo);

	if (!(iCameraInfo.iFlashModesSupported & CCamera::EFlashVideoLight))
		User::Leave(KErrNotSupported);

	// retry if error occurred previously
	if (iState == ECameraErr)
		{
		iState = ECameraIdle;
		}

	if (aOn && iState == ECameraIdle)
		{
		iState = ECameraInit;
		iCamera->Reserve();
		}
	else if (!aOn && iState == ECameraRun)
		{
		StopCapture();
		}
	}

// Handle events related to reserving and powering on the camera
void CCameraFlashLight::HandleEvent(const TECAMEvent &aEvent)
	{
	if (aEvent.iErrorCode != KErrNone)
		{
		iState = ECameraErr;
		return;
		}

	if (aEvent.iEventType == KUidECamEventReserveComplete)
		iCamera->PowerOn();

	if (aEvent.iEventType == KUidECamEventPowerOnComplete)
		{
		TRAPD(err, PrepareAndStartCaptureL());
		if (err)
			iState = ECameraErr;
		}
	}

void CCameraFlashLight::PrepareAndStartCaptureL()
	{
	iCamera->SetFlashL(CCamera::EFlashVideoLight);

	// Use the last listed capture frame size - that should give us
	// the smallest frames (doesn't matter that much, as they are 
	// not actually used for anything.
	iCamera->PrepareVideoCaptureL(CCamera::EFormatYUV420Planar,
			iCameraInfo.iNumVideoFrameSizesSupported - 1, 0, 2, 1);
	iCamera->StartVideoCapture();
	iState = ECameraRun;
	}

void CCameraFlashLight::StopCapture()
	{
	iCamera->SetFlashL(CCamera::EFlashNone);
	iCamera->StopVideoCapture();
	
	iCamera->PowerOff();
	iCamera->Release();
	iState = ECameraIdle;
	}

void CCameraFlashLight::ViewFinderReady(MCameraBuffer&, TInt)
	{
	}
void CCameraFlashLight::ImageBufferReady(MCameraBuffer&, TInt)
	{
	}
void CCameraFlashLight::VideoBufferReady(MCameraBuffer& aBuf, TInt aError)
	{
	// simply release the buffer
	if (aError == KErrNone)
		aBuf.Release();
	}

// eof
